--[[
YAGRA - gear rating addon
Copyright 2010 Adirelle (adirelle@tagada-team.net)
All rights reserved.
--]]

local addonName, addon = ...

local A, B, C, D = 1, 3/4, 9/16, 81/256

local INVTYPE_MODIFIER = {
	INVTYPE_HEAD           = A,
	INVTYPE_CHEST          = A,
	INVTYPE_ROBE           = A,
	INVTYPE_LEGS           = A,

	INVTYPE_SHOULDER       = B,
	INVTYPE_WAIST          = B,
	INVTYPE_FEET           = B,
	INVTYPE_HAND           = B,

	INVTYPE_TRINKET        = C,
	INVTYPE_NECK           = C,
	INVTYPE_WRIST          = C,
	INVTYPE_FINGER         = C,
	INVTYPE_CLOAK          = C,

	-- Sum of weapon(s) and relic/ranged is 2*A+D
	INVTYPE_2HWEAPON       = 2*A,
	INVTYPE_WEAPONMAINHAND = A,
	INVTYPE_WEAPONOFFHAND  = A,
	INVTYPE_SHIELD         = A,
	INVTYPE_WEAPON         = A,
	INVTYPE_HOLDABLE       = A,

	INVTYPE_RELIC          = D,
	INVTYPE_RANGED         = D,
	INVTYPE_THROWN         = D,
	INVTYPE_RANGEDRIGHT    = D,
}

-- These modifiers are applied after INVTYPE_MODIFIER
-- Weapon mods are swapped for hunters so ranged weapons are more important
local HUNTER_MOD = {
	INVTYPE_RANGED = (2*A-D)/D,
	INVTYPE_RANGEDRIGHT = (2*A-D)/D,
	INVTYPE_2HWEAPON = D,
	INVTYPE_WEAPONMAINHAND = D,
	INVTYPE_WEAPONOFFHAND = D,
	INVTYPE_WEAPON = D,
	INVTYPE_HOLDABLE = D,
}

-- Items concerned by altenative rating
local ALTERNATIVE_ITEM_TYPES = {
	INVTYPE_WEAPONMAINHAND = true,
	INVTYPE_WEAPONOFFHAND  = true,
	INVTYPE_SHIELD         = true,
	INVTYPE_WEAPON         = true,
	INVTYPE_HOLDABLE       = true,
}
local ALTERNATIVE_SLOT_MODS = {
	[INVSLOT_MAINHAND] = 5/4,
	[INVSLOT_OFFHAND]  = 3/4,
}

-- Total sum of weights for any class
local MODIFIER_SUM = 5*A+4*B+7*C+D

local QUALITY_DATA = {
	[0] = { scale = 0.005, key = "UNCOMMON" }, -- Poor
	[1] = { scale = 0.005, key = "UNCOMMON" }, -- Common
	[2] = { scale = 1,     key = "UNCOMMON" }, -- Uncommon
	[3] = { scale = 1,     key = "RARE" }, -- Rare
	[4] = { scale = 1,     key = "EPIC" }, -- Epic
	[5] = { scale = 1.3,   key = "EPIC" }, -- Legendary
	-- [6] -- Artifact
	[7] = { scale = 1,     key = "RARE", fakeItemLevel = 187.5 }, -- Heirloom
}

local VALUE_CONSTS = {
	VANILLA = {
		UNCOMMON = { k = 8.0000, f = 2.0000 },
		RARE     = { k = 0.7500, f = 1.8000 },
		EPIC     = { k = 26.000, f = 1.2000 },
	},
	BC_AND_WOTLK = {
		UNCOMMON = { k = 73.0000, f = 1.0000 },
		RARE     = { k = 81.3750, f = 0.8125 },
		EPIC     = { k = 91.4500, f = 0.6500 },
	},
	CATACLYSM = {
		UNCOMMON = { k = 91.4500, f = 0.6500 },
		RARE     = { k = 91.4500, f = 0.6500 },
		EPIC     = { k = 91.4500, f = 0.6500 },
	},
}

local BASE_SCALE = 1.8618

local RATING_WEAPON_ALTERNATIVE_FLAG = 0x1
local RATING_LEGACY_QUALITY_FLAGS = 0x2

addon.RATING_WEAPON_ALTERNATIVE_FLAG = RATING_WEAPON_ALTERNATIVE_FLAG
addon.RATING_LEGACY_QUALITY_FLAGS = RATING_LEGACY_QUALITY_FLAGS

local function MyGetItemInfo(item)
	if not item or item == "0:0" then return end
	return GetItemInfo("item:"..item)
end

local function GetScoreForItemLevel(level, quality)
	local qualityData = QUALITY_DATA[quality or ITEM_QUALITY_EPIC]
	if qualityData then
		local baseLevel = qualityData.fakeItemLevel or level
		local itemExpansion = "BC_AND_WOTLK"
		if baseLevel < 120 then
			itemExpansion = "VANILLA"
		elseif not addon.db.profile.legacyQualityModifier and baseLevel > 277 then
			itemExpansion = "CATACLYSM"
		end
		local consts = VALUE_CONSTS[itemExpansion][qualityData.key]
		return BASE_SCALE * (baseLevel - consts.k) / consts.f * qualityData.scale
	else
		return 0
	end
end

addon.GetScoreForFullStuff = function(level, quality)
	return GetScoreForItemLevel(level, quality) * MODIFIER_SUM
end

local statTable = {}
function addon:GetBaseItemScore(item)
	local name, link, quality, level, _, _, _, _, invType = MyGetItemInfo(item)
	if name and quality and level and invType then
		local score = GetScoreForItemLevel(level, quality) * (INVTYPE_MODIFIER[invType] or 1)
		wipe(statTable)
		GetItemStats(link, statTable)
		return score, level, invType, statTable.ITEM_MOD_RESILIENCE_RATING_SHORT or 0
	end
end

function addon:GetItemScore(item, isHunter, withTitansGrip, slot, alternativeRating)
	local score, level, invType, resilience = self:GetBaseItemScore(item)
	if score then
		if alternativeRating and slot and ALTERNATIVE_ITEM_TYPES[invType] and ALTERNATIVE_SLOT_MODS[slot] then
			score = score * ALTERNATIVE_SLOT_MODS[slot]
		end
		local hunterMod = isHunter and HUNTER_MOD[invType]
		if hunterMod then
			score = score * hunterMod
		elseif withTitansGrip and invType == "INVTYPE_2HWEAPON" then
			score = score * 0.5
		end
	end
	return score, level, invType, resilience
end

function addon:GetPlayerItemScore(record, item, slot, alternativeRating)
	if not item or item == "0:0" then return end
	return self:GetItemScore(
		item,
		(record.class == "HUNTER"),
		(record.class == "WARRIOR" and select(9, MyGetItemInfo(record[INVSLOT_OFFHAND])) == "INVTYPE_2HWEAPON"),
		slot,
		alternativeRating
	)
end

function addon:GetPlayerScore(record, alternativeRating)
	local score, level, resilience, count = 0, 0, 0, 0
	for slot = 1, 18 do
		local itemScore, itemLevel, _, itemResilience = self:GetPlayerItemScore(record, record[slot], slot, alternativeRating)
		if itemScore then
			score = score + itemScore
			level = level + itemLevel
			resilience = resilience + itemResilience
			count = count + 1
		end
	end
	self:Debug(alternativeRating and 'Alternative' or 'Compat', 'rating of', record.name, ': score=', score, 'sum of levels=', level, 'resilience=', resilience, '#items=', count)
	return floor(score + 0.5), floor(level / count + 0.5), resilience
end

function addon:UpdatePlayerScore(record, source, quiet)
	local alternativeRating = self.db.profile.alternativeRating
	local ratingScheme = self:GetRatingSchemeID()
	local legacyQualityModifier = self.db.profile.legacyQualityModifier
	local score, average, resilience = self:GetPlayerScore(record, alternativeRating)
	if record.score ~= score or record.average ~= average or record.resilience ~= resilience or record.ratingScheme ~= ratingScheme then
		record.score, record.average, record.resilience, record.ratingScheme = score, average, resilience, ratingScheme
		self:Debug('Updated score, average level and resilience of', record.name, ',', alternativeRating and "alernative" or "compat", 'score:', score, average, resilience)
		if not quiet then
			self:SendMessage('YAGRA_RecordUpdated', record, source)
		end
	end
end

function addon:GetRatingSchemeID()
	return bit.bor(
		self.db.profile.alternativeRating and RATING_WEAPON_ALTERNATIVE_FLAG or 0,
		self.db.profile.legacyQualityModifier and RATING_LEGACY_QUALITY_FLAGS or 0
	)
end
