--[[
YAGRA - gear rating addon
Copyright 2010 Adirelle (adirelle@tagada-team.net)
All rights reserved.
--]]

local addonName, addon = ...
local L = addon.L

local playerName, playerRealm = addon.playerName, addon.playerRealm
local Colorize, ClassColorizedName, UnitNameRealm = addon.Colorize, addon.ClassColorizedName, addon.UnitNameRealm

local LibDBIcon = LibStub('LibDBIcon-1.0')

local mod = addon:NewModule('Broker', 'AceTimer-3.0')
mod.enabledInCombat = true

local dataobj = {
	type = 'data source',
	text = addonName,
	icon = [[Interface\Icons\Spell_Nature_EnchantArmor]],
}

local inCombat = false

function mod:OnInitialize()
	self:RegisterDatabase({
		profile = {
			mode = 'history',
			minimapIcon = {
				hide = false
			},
		}
	})
	LibStub('LibDataBroker-1.1'):NewDataObject(addonName, dataobj)
	LibDBIcon:Register(addonName, dataobj, self.db.profile.minimapIcon)
end

function mod:OnEnable()
	self:RegisterMessage('YAGRA_RecordUpdated')
	self:RegisterMessage('YAGRA_ConfigChanged', "Update")
	self:RegisterEvent('PARTY_MEMBERS_CHANGED')
	self:RegisterEvent('GUILD_ROSTER_UPDATE')
	self:RegisterEvent('PLAYER_REGEN_DISABLED')
	self:RegisterEvent('PLAYER_REGEN_ENABLED')
	inCombat = InCombatLockdown()
	dataobj.label = addonName
	self:Update('OnEnable')
end

function mod:OnDisable()
	dataobj.label = nil
	dataobj.text = addonName
	self:HideTooltip('OnDisable')
end

function mod:GetOptions()
	return {
		minimapIcon = {
			type = 'toggle',
			name = L['Show minimap icon'],
			get = function() return not self.db.profile.minimapIcon.hide end,
			set = function(_, value)
				self.db.profile.minimapIcon.hide = not value
				if value then
					LibDBIcon:Show(addonName)
				else
					LibDBIcon:Hide(addonName)
				end
			end,
		},
	}, self:GetOptionHandler()
end

local history = {}
function mod:YAGRA_RecordUpdated(event, record, source)
	local name = record.fullName
	for i = 1, #history do
		if history[i] == name then
			tremove(history, i)
			break
		end
	end
	tinsert(history, 1, name)
	history[21] = nil -- Hard way to restrict to 20 entries
	return self:Update(event)
end

function mod:PARTY_MEMBERS_CHANGED(event)
	if not inCombat and self.db.profile.mode == 'group' then 
		return self:Update(event) 
	end
end

function mod:GUILD_ROSTER_UPDATE()
	if not inCombat and self.db.profile.mode == 'guild' then 
		return self:UpdateTooltip(event) 
	end	
end

function mod:PLAYER_REGEN_DISABLED(event)
	inCombat = true
	return self:HideTooltip(event)
end

function mod:PLAYER_REGEN_ENABLED(event)
	inCombat = false
	return self:Update(event)
end

function mod:Update(event)
	if inCombat then return end
	self:Debug('Updating on', event)
	local record = addon:GetPlayerRecord(playerName, playerRealm)
	if record then
		dataobj.text = Colorize(addon:FormatScore(record, true, true))
	else
		dataobj.text = '-'
	end
	self:UpdateTooltip(event)
end

local cycle = {
	history = "group",
	group = "guild",
	guild = "history"
}

function mod:CycleMode()
	self.db.profile.mode = cycle[self.db.profile.mode] or history
	self:Debug('New display mode:', self.db.profile.mode)
	self:Update('CycleMode')
end

--------------------------------------------------------------------------------
-- Tooltip stuff
--------------------------------------------------------------------------------

local tooltip

local function AddScore(record)
	local score, r, g, b, age = addon:FormatScore(record, true, true)
	tooltip:AddLine(ClassColorizedName(record), record.level, Colorize(score, r, g, b), age)
end

local function AddFullLine(text, isHeader)
	local y = tooltip:AddLine()
	tooltip:SetCell(y, 1, text, isHeader and tooltip:GetHeaderFont() or nil, isHeader and "CENTER" or "LEFT", 4)
	return y
end

local function AddTableHeader(text)
	AddFullLine(text, true)
	tooltip:AddLine(L["Name"], L["Level"], L["Score"], L["Last updated"])
	tooltip:AddSeparator()
end

local function SortByTimestamp(a, b)
	return a.timestamp > b.timestamp
end

local function SortByName(a, b)
	return a.fullName < b.fullName
end

local function SortByScore(a, b)
	return a.score > b.score
end

local modeHandlers = {

	group = function(names)
		local prefix, last, first = "raid", GetNumRaidMembers(), 1
		if last == 0 then
			prefix, last, first = "party", GetNumPartyMembers(), 0
		end
		for i = first, last do
			local unit = (i == 0) and "player" or (prefix..i)
			if UnitExists(unit) then
				local _, _, fullName = UnitNameRealm(unit)
				tinsert(names, fullName)
			end
		end
		return L['Group'], names, SortByName
	end,
	
	history = function()
		return L["Recently updated characters"], history, SortByTimestamp
	end,

	guild = function(names)
		for index = 1, GetNumGuildMembers(true) do
			local name, _, _, level = GetGuildRosterInfo(index)
			if level >= addon.db.profile.minLevel then
				tinsert(names, name)
			end
		end
		return L["Guild"], names, SortByScore
	end,

}

local QTip = LibStub('LibQTip-1.0')

local function Tooltip_OnRelease(self, ...)
	assert(self:IsAcquiredBy(mod.name))
	mod:CancelTimer(mod.updateTimer, true)
	tooltip = nil
end

function mod:ShowTooltip(event, anchor)
	if inCombat then return end
	self:Debug('ShowTooltip', event, anchor)
	if not tooltip then
		tooltip = QTip:Acquire(self.name, 4, "LEFT", "RIGHT", "RIGHT", "RIGHT")
		tooltip:SetScript('OnHide', tooltip.Release)
		tooltip.OnRelease = Tooltip_OnRelease
		self.updateTimer = self:ScheduleRepeatingTimer('UpdateTooltip', 1, 'ScheduledUpdate')
	else
		assert(tooltip:IsAcquiredBy(self.name))
	end
	tooltip:SetAutoHideDelay(0.25, anchor)
	tooltip:SmartAnchorTo(anchor)
	self:UpdateTooltip(event)
end

function mod:HideTooltip(event)
	if tooltip then
		assert(tooltip:IsAcquiredBy(self.name))
		tooltip:Hide()
	end
end

local tmpNames = {}
local records = {}
function mod:UpdateTooltip(event)
	if inCombat or not tooltip then return end
	assert(tooltip:IsAcquiredBy(self.name))
	self:Debug('Updating the tooltip on', event)	
	tooltip:Clear()
	wipe(tmpNames)
	local title, names, sortFunc = modeHandlers[self.db.profile.mode](tmpNames)
	AddTableHeader(title)
	if #names == 0 then
		AddFullLine(L["No entries"])
	else
		wipe(records)
		for i, name in ipairs(names) do
			local record = addon:GetPlayerRecord(name)
			if record then
				tinsert(records, record)
			end
		end
		table.sort(records, sortFunc)
		for i, record in ipairs(records) do
			AddScore(record)
		end
	end
	tooltip:AddSeparator()
	AddFullLine(L["Left-click to change tooltip data"])
	AddFullLine(L["Right-click to open the configuration panel"])
	tooltip:Show()
	tooltip:UpdateScrolling(UIParent:GetHeight()/3)
end

--------------------------------------------------------------------------------
-- Data object scripts
--------------------------------------------------------------------------------

function dataobj.OnClick(frame, button)
	if button == "RightButton" then
		InterfaceOptionsFrame_OpenToCategory(addon.optionPanel)
	else
		mod:CycleMode()
	end
end

function dataobj.OnEnter(anchor)
	return mod:ShowTooltip('dataobj.OnEnter', anchor)
end

